package org.dasein.cloud.jclouds.terremark;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dasein.cloud.AsynchronousTask;
import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.OperationNotSupportedException;
import org.dasein.cloud.services.image.ImageServices;
import org.dasein.cloud.services.image.ServerImage;
import org.dasein.cloud.services.server.Architecture;
import org.dasein.cloud.services.server.Platform;
import org.dasein.cloud.services.server.ServerSize;
import org.dasein.cloud.services.storage.CloudStoreObject;
import org.jclouds.rest.RestContext;
import org.jclouds.vcloud.domain.ReferenceType;
import org.jclouds.vcloud.domain.VCloudExpressVAppTemplate;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressAsyncClient;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressClient;

public class Template implements ImageServices {
    private Terremark provider;
    
    Template(Terremark provider) {
        this.provider = provider;
    }
    
    @Override
    public String create(String templateId) throws InternalException, CloudException {
        throw new OperationNotSupportedException();
    }

    @Override
    public void delete(String templateId) throws InternalException, CloudException {
        throw new OperationNotSupportedException();
    }

    @Override
    public Collection<ServerImage> find(String executableByAccountNumber, String keyword, Platform platform, String architecture) throws InternalException, CloudException {
        ArrayList<ServerImage> matches = new ArrayList<ServerImage>();
        
        for( ServerImage image : list(provider.getContext().getAccountNumber()) ) {
            if( keyword != null ) {
                if( image.getName().indexOf(keyword) == -1 ) {
                    if( image.getDescription().indexOf(keyword) == -1 ) {
                        continue;
                    }
                }
            }
            if( platform != null ) {
                if( !platform.equals(image.getPlatform()) ) {
                    if( platform.equals(Platform.UNIX) ) {
                        if( !image.getPlatform().isUnix() ) {
                            continue;
                        }
                    }
                    else if( platform.equals(Platform.WINDOWS) ) {
                        if( !image.getPlatform().isWindows()) {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
            if( architecture != null ) {
                if( !architecture.equals(image.getArchitecture().name()) ) {
                    continue;
                }
            }
            matches.add(image);
        }
        return matches;
    }

    private Architecture getArchitecture(String templateName) {
        if( templateName == null || templateName.indexOf("32-bit") != -1 || templateName.indexOf("32 bit") != -1 ) {
            return Architecture.I32;            
        }
        else if( templateName.indexOf("64-bit") != -1 || templateName.indexOf("64 bit") != -1 ) {
            return Architecture.I64;
        }
        return Architecture.I64;
    }
    
    @Override
    public ServerImage getImage(String templateId) throws InternalException, CloudException {
        for( ServerImage image : list(provider.getContext().getAccountNumber()) ) {
            if( image.getProviderServerImageId().equals(templateId) ) {
                return image;
            }
        }
        return null;
    }

    @Override
    public CloudStoreObject getManifest(String templateId) throws CloudException, InternalException {
        return null;
    }

    static private final Pattern USER_PASSWORD_PATTERN = Pattern.compile(".*[Uu]sername: ([a-z]+) ?.*\n[Pp]assword: ([^ ]+) ?\n.*");

    String getPassword(String id) {
        RestContext<TerremarkVCloudExpressClient,TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();
        VCloudExpressVAppTemplate template = null;
                
        try {
            template = ctx.getApi().getVAppTemplate(URI.create(id));            
        }
        finally {
            ctx.close();
        }
        if (template.getDescription().indexOf("Windows") >= 0) {
            return "Administrator:null";
        } 
        else {
            Matcher matcher = USER_PASSWORD_PATTERN.matcher(template.getDescription());
            
            if( matcher.find() ) {
                String pw = matcher.group(1) + ":" + matcher.group(2);
                
                return pw;
            }
            else {
                return "UNKNOWN";
            }
        }
    }
    
    @Override
    public String getProviderTermForImage(Locale locale) {
        return "template";
    }

    @Override
    public Collection<String> getShares(String templateId) throws CloudException, InternalException {
        return new ArrayList<String>();
    }

    @Override
    public AsynchronousTask<ServerImage> imageServer(String serverId, String forAccountAccessKey, String directory, String imageName) throws InternalException, CloudException {
        throw new OperationNotSupportedException();
    }

    @Override
    public boolean isPublic(String templateId) throws InternalException, CloudException {
        return true;
    }

    @Override
    public boolean isSupportsCustomImages() throws InternalException, CloudException {
        return false;
    }

    @Override
    public boolean isSupportsSharing() throws InternalException, CloudException {
        return false;
    }

    @Override
    public Collection<ServerImage> list(String accountOwner) throws InternalException, CloudException {
        RestContext<TerremarkVCloudExpressClient,TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();
        
        try {
            ArrayList<ServerImage> images = new ArrayList<ServerImage>();
            Collection<ReferenceType> resources;
            
            
            resources = ctx.getApi().findCatalogInOrgNamed(null, null).values();
            for( ReferenceType rsc : resources ) {
                ServerImage image = toImage(rsc);
                
                if( image != null ) {
                    images.add(image);
                }
            }
            return images;
        }
        finally {
            ctx.close();
        }
    }
    
    private ServerSize[] getSizes() {
        return new ServerSize[] { new ServerSize("512:1", "1 CPU, 512M RAM, 4GB Disk") };
    }

    @Override
    public void share(String templateId, String accountId, boolean allowed) throws InternalException, CloudException {
        throw new OperationNotSupportedException();
    }

    private ServerImage toImage(ReferenceType rsc) {
        ServerImage image = new ServerImage();
        
        image.setActive(true);
        image.setArchitecture(getArchitecture(rsc.getName()));
        image.setAvailable(true);
        image.setDescription(rsc.getName());
        image.setName(rsc.getName());
        image.setOwnerAccountId("--terremark--");
        image.setPersistent(true);
        image.setPlatform(Platform.guess(rsc.getName()));
        image.setProviderServerImageId(rsc.getHref().toASCIIString());
        image.setPublicImage(true);
        image.setSize(getSizes());
        image.setType("standard");
        return image;
    }
}
