package org.dasein.cloud.jclouds.terremark;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.services.dc.DataCenter;
import org.dasein.cloud.services.dc.DataCenterServices;
import org.dasein.cloud.services.dc.Region;
import org.jclouds.rest.RestContext;
import org.jclouds.vcloud.domain.VDC;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressAsyncClient;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressClient;

public class Vdc implements DataCenterServices {

    private transient Terremark provider;
    
    Vdc(Terremark provider) {
        this.provider = provider;
    }
    
    @Override
    public DataCenter getDataCenter(String dataCenterId) throws InternalException, CloudException {
        VDC vdc = getVdc(dataCenterId);
        
        if( vdc != null ) {
            DataCenter dc = new DataCenter();
            
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName(vdc.getName());
            dc.setProviderDataCenterId(dataCenterId);
            dc.setRegionId(provider.getContext().getRegionId());
            return dc;
        }
        return null;
    }

    @Override
    public String getProviderTermForDataCenter(Locale locale) {
        return "zone";
    }

    @Override
    public String getProviderTermForRegion(Locale locale) {
        return "VDC";
    }

    @Override
    public Region getRegion(String regionId) throws InternalException, CloudException {
        if( regionId == null || !regionId.equals("us-miami") ) {
            return null;
        }
        Region region = new Region();
        
        region.setActive(true);
        region.setAvailable(true);
        region.setName("US/Miami");
        region.setProviderRegionId("us-miami");
        return region;
    }

    VDC getVdc(String vdcId) {
        RestContext<TerremarkVCloudExpressClient, TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();

        try {
            VDC vdc = ctx.getApi().findVDCInOrgNamed(null, null);
        
            if( vdc == null ) {
                return null;
            }
            if( !vdc.getHref().toASCIIString().equals(vdcId) ) {
                return null;
            }
            return vdc;
        }
        finally {
            ctx.close();
        }
    }
    
    @Override
    public Collection<DataCenter> listDataCenters(String regionId) throws InternalException, CloudException {
        ArrayList<DataCenter> dataCenters = new ArrayList<DataCenter>();
        
        if( regionId == null || !regionId.equals("us-miami") ) {
            return dataCenters;
        }
        RestContext<TerremarkVCloudExpressClient,TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();

        try {
            VDC vdc = ctx.getApi().findVDCInOrgNamed(null, null);
            DataCenter dc = new DataCenter();
            
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName(vdc.getName());
            dc.setRegionId("us-miami");
            dc.setProviderDataCenterId(vdc.getHref().toASCIIString());
            dataCenters.add(dc);
        }
        finally {
            ctx.close();
        }
        return dataCenters;
    }

    @Override
    public Collection<Region> listRegions() throws InternalException, CloudException {
        ArrayList<Region> regions = new ArrayList<Region>();

        regions.add(getRegion("us-miami"));
        return regions;
    }

    @Override
    public String matchDataCenter(CloudProvider provider, String dataCenterId) throws InternalException, CloudException {
        return null;
    }

}
